from __future__ import annotations
from typing import Union
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..client import PlunetClient
from ..enums import FolderType


class DataUser30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_user_by_id(self, user_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataUser30.getUserByID, user_id
        )

    def get_user_by_login_name(self, login_name: str):
        return self.__client.make_request(
            self.__client.plunet_server.DataUser30.getUserByLoginName, login_name
        )

    def get_user_list_by_resource_id(self, resource_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataUser30.getUserListByResourceID, resource_id
        )


class DataDocument30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_file_list(self, main_id: int, folder_type: Union[FolderType, int]):
        """

        :param main_id: Int id for entity type: OrderID, JobID, RequestID, etc.
        :param folder_type:
        :return:
        """
        if type(folder_type) == int:
            arg = {"MainID": main_id, "FolderType": folder_type}
        else:
            arg = {"MainID": main_id, "FolderType": folder_type.value}
        return self.__client.make_request(
            self.__client.plunet_server.DataDocument30.getFileList,
            arg,
            unpack_dict=True,
        )

    def download_document(
        self, main_id: int, folder_type: Union[FolderType, int], file_path_name: str
    ):
        """

        :param main_id: Int id for entity type: OrderID, JobID, RequestID, etc.
        :param folder_type:
        :param file_path_name:
        :return:
        """
        arg = {"MainID": main_id, "FilePathName": file_path_name}
        if type(folder_type) == int:
            arg["FolderType"] = folder_type
        else:
            arg["FolderType"] = folder_type.value
        return self.__client.make_download(
            self.__client.plunet_server.DataDocument30.download_Document,
            arg,
            unpack_dict=True,
        )

    def upload_document(
        self,
        main_id: int,
        folder_type: Union[FolderType, int],
        file_path_name: str,
        file_size: int,
        byte_stream: bytes,
    ):

        arg = {
            "FileByteStream": byte_stream,
            "FilePathName": file_path_name,
            "FileSize": file_size,
            "MainID": main_id,
        }

        if type(folder_type) == int:
            arg["FolderType"] = folder_type
        else:
            arg["FolderType"] = folder_type.value

        return self.__client.make_request(
            self.__client.plunet_server.DataDocument30.upload_Document,
            arg,
            unpack_dict=True,
        )



from __future__ import annotations
from typing import List, Union

from ..models import SearchFilter_Order, SearchFilter_Quote, SearchFilter_Request

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..client import PlunetClient

from ..enums import ProjectType


class DataOrder30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_order_language_combinations(self, order_id: int):
        """
        TODO: check return type
        :param order_id:
        :return:
        """
        return self.__client.make_request(
            self.__client.plunet_server.DataOrder30.getLanguageCombination, order_id
        )

    def get_order_id_from_display_string_name(self, order_display_number: str) -> int:
        """
        Takes the order number string, f.ex. O-00123, and returns the order id as an int.

        :param order_display_number:
        :return:
        """
        return self.__client.make_request(
            self.__client.plunet_server.DataOrder30.getOrderID, order_display_number
        )

    def get_order_creation_date(self, order_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataOrder30.getCreationDate, order_id
        )

    def search_for_orders(
        self, search_filter: Union[SearchFilter_Order, dict]
    ) -> List[int]:
        if type(search_filter) != SearchFilter_Order:
            arg = SearchFilter_Order(**search_filter).dict()
        else:
            arg = search_filter.dict()
        data_item = self.__client.make_request(
            self.__client.plunet_server.DataOrder30.search, arg
        )
        return data_item

    def get_order_object(self, order_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataOrder30.getOrderObject, order_id
        )

    def get_order_object_list(self, order_object_ids: List[int]):
        arg = self.__client.plunet_server.DataOrder30.factory.IntegerList(
            order_object_ids
        )
        return self.__client.make_request(
            self.__client.plunet_server.DataOrder30.getOrderObjectList, arg
        )


class DataItem30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_order_item_id_by_language_combi(
        self,
        order_id: int,
        source_lang: str,
        target_lang: str,
        project_type: ProjectType = ProjectType.ORDER,
    ):
        arg = {
            "projectType": project_type.value,
            "projectID": order_id,
            "sourceLanguage": source_lang,
            "targetLanguage": target_lang,
        }

        return self.__client.make_request(
            self.__client.plunet_server.DataItem30.get_ByLanguage, arg, unpack_dict=True
        )

    def get_all_order_item_objects(
        self, order_id: int, project_type: ProjectType = ProjectType.ORDER
    ):

        arg = {"projectType": project_type.value, "projectID": order_id}

        return self.__client.make_request(
            self.__client.plunet_server.DataItem30.getAllItemObjects,
            arg,
            unpack_dict=True,
        )


class DataQuote30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def search_for_quotes(self, search_filter: Union[SearchFilter_Quote, dict]):
        if type(search_filter) != SearchFilter_Quote:
            arg = SearchFilter_Quote(**search_filter).dict()
        else:
            arg = search_filter.dict()

        return self.__client.make_request(
            self.__client.plunet_server.DataQuote30.search, arg
        )

    def get_quote_object_list(self, quote_id_list: List[int]):
        arg = self.__client.plunet_server.DataQuote30.factory.IntegerList(quote_id_list)
        return self.__client.make_request(
            self.__client.plunet_server.DataQuote30.getQuoteObjectList, arg
        )

    def get_quote_object(self, quote_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataQuote30.getQuoteObject, quote_id
        )

    def get_quote_customer_contact_person(self, quote_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataQuote30.getCustomerContactID, quote_id
        )


class DataRequest30:
    def __init__(self, client: PlunetClient):
        self.client = client

    def search_for_requests(self, search_filter: Union[SearchFilter_Request, dict]):
        if type(search_filter) != SearchFilter_Request:
            arg = SearchFilter_Request(**search_filter).dict()
        else:
            arg = search_filter.dict()
        return self.client.make_request(
            self.client.plunet_server.DataRequest30.search, arg
        )

    def get_request_object_list(self, request_id_list: List[int]):
        arg = self.client.plunet_server.DataRequest30.factory.IntegerList(
            request_id_list
        )
        return self.client.make_request(
            self.client.plunet_server.DataRequest30.getRequestObjectList, arg
        )

    def get_request_customer_contact_person(self, request_id: int):
        return self.client.make_request(
            self.client.plunet_server.DataRequest30.getCustomerContactID, request_id
        )



class PlunetException(Exception):
    pass


class PlunetAuthFailed(PlunetException):
    pass


class PlunetOrderNotFound(PlunetException):
    pass


class NoPlunetSession(PlunetException):
    pass



from .admin_api import DataAdmin30
from .internal_api import DataDocument30, DataUser30
from .invoice_api import DataOutgoingInvoice30
from .job_api import DataJob30, ReportJob30
from .partner_api import DataCustomer30, DataCustomerContact30, ReportCustomer30
from .project_api import DataItem30, DataOrder30, DataQuote30, DataRequest30

__version__ = "0.0.1"
__all__ = [
    DataAdmin30,
    DataDocument30,
    DataUser30,
    DataOutgoingInvoice30,
    DataJob30,
    ReportJob30,
    DataCustomer30,
    DataCustomerContact30,
    ReportCustomer30,
    DataOrder30,
    DataQuote30,
    DataItem30,
    DataRequest30,
]



from datetime import datetime
from typing import Any, List, Optional, Union

from pydantic import BaseModel as BM
from pydantic import Field

from .enums import (
    ContactPersonStatus,
    ExportedType,
    InvoiceStatusType,
    PayableStatus,
    ResourceType,
    SearchScope,
    SearchSelection_Customer,
    TimeFrameRelation_Invoice,
    TimeFrameRelation_Quote,
    TimeFrameRelation_Request,
)


class BaseModel(BM):
    class Config:
        use_enum_values = True


class SearchFilter_Customer(BaseModel):
    contact_resourceID: str = "-1"
    customerType: str = "-1"
    email: Optional[str] = None
    languageCode: str = "EN"
    name1: Optional[str] = None
    name2: Optional[str] = None
    propertiesList: List[Any] = Field(default_factory=list)
    sourceLanguageCode: Optional[str] = None
    customerStatus: str = "-1"
    textmodulesList: List[Any] = Field(default_factory=list)


class SelectionEntry_TeamMember(BaseModel):
    ID: int
    teamMember: ResourceType  # resourceEntryType


class SelectionEntry_Customer(BaseModel):
    customerEntryType: Optional[Union[SearchSelection_Customer, int]] = None
    mainID: int


class SelectionEntry_TimeFrame(BaseModel):
    dateFrom: datetime
    dateRelation: Union[
        SearchScope,
        TimeFrameRelation_Invoice,
        TimeFrameRelation_Quote,
        TimeFrameRelation_Request,
    ]
    dateTo: datetime


class SearchFilter_Order(BaseModel):
    customerID: int = -1
    itemStatus: List[Any] = Field(default_factory=list)
    languageCode: str = "EN"
    orderStatus: int = -1
    projectDescription: Optional[str] = None
    projectName: Optional[str] = None
    projectType: int = -1
    propertiesList: List[Any] = Field(default_factory=list)
    sourceLanguage: Optional[str] = None  # ISO 639-code
    statusProjectFileArchiving: int = -1
    targetLanguage: Optional[str] = None
    teamMember: Optional[SelectionEntry_TeamMember] = None
    textmodulesList: List[Any] = Field(default_factory=list)
    timeFrame: Optional[SelectionEntry_TimeFrame] = None


class SearchFilter_Quote(BaseModel):
    languageCode: str = "EN"
    propertiesList: List[Any] = Field(default_factory=list)
    selectionEntryCustomer: Optional[SelectionEntry_Customer] = None
    SelectionEntry_Resource: Optional[Any] = None  # TODO
    SelectionEntry_TeamMember: Optional[SelectionEntry_TeamMember] = None
    sourceLanguage: Optional[str] = None
    quoteStatus: Optional[int] = -1
    targetLanguage: Optional[str] = None
    textmodulesList: List[Any] = Field(default_factory=list)
    timeFrame: Optional[SelectionEntry_TimeFrame] = None


class SearchFilter_Job(BaseModel):
    customerID: Optional[int] = -1
    item_Status: Optional[int] = -1
    job_CreationDate_from: Optional[datetime] = None
    job_CreationDate_to: Optional[datetime] = None
    job_SourceLanguage: Optional[str] = None
    job_Status: Optional[int] = -1
    job_TargetLanguage: Optional[str] = None
    job_resourceID: Optional[int] = -1


class SearchFilter_Invoice(BaseModel):
    invoiceStatus: Optional[InvoiceStatusType] = "-1"
    customerID: Optional[str] = "-1"
    propertiesList: List[Any] = Field(default_factory=list)
    languageCode: Optional[str] = "EN"
    textmodulesList: List[Any] = Field(default_factory=list)
    timeFrame: Optional[SelectionEntry_TimeFrame] = None


class SearchFilter_Payable(BaseModel):
    exported: ExportedType = 3
    payableStatus: Optional[PayableStatus] = -1
    resourceID: Optional[int] = -1
    languageCode: Optional[str] = "EN"
    isoCodeCurrency: Optional[str] = None
    propertiesList: List[Any] = Field(default_factory=list)
    textmodulesList: List[Any] = Field(default_factory=list)
    timeFrame: Optional[SelectionEntry_TimeFrame] = None


class SearchFilter_Request(BaseModel):
    languageCode: str = "EN"
    propertiesList: List[Any] = Field(default_factory=list)
    selectionEntry_Customer: Optional[SelectionEntry_Customer] = None
    sourceLanguage: Optional[str] = None
    requestStatus: Optional[int] = -1
    targetLanguage: Optional[str] = None
    textmodulesList: List[Any] = Field(default_factory=list)
    timeFrame: Optional[SelectionEntry_TimeFrame] = None


class OrderIN(BaseModel):
    currency: Optional[Any] = None
    customerContactID: Optional[Any] = None
    customerID: Optional[Any] = None
    deliveryDeadline: Optional[Any] = None
    orderDate: Optional[Any] = None
    orderID: Optional[Any] = None
    projectManagerID: Optional[Any] = None
    projectManagerMemo: Optional[Any] = None
    projectName: Optional[Any] = None
    rate: Optional[Any] = None
    referenceNumber: Optional[Any] = None
    subject: Optional[Any] = None


class CustomerObject(BaseModel):
    academicTitle: Optional[str] = None
    costCenter: Optional[str] = None
    currency: Optional[str] = None
    customerID: int
    email: str
    externalID: Optional[str] = None
    fax: Optional[str] = None
    formOfAddress: int
    fullName: str
    mobilePhone: Optional[str] = None
    name1: Optional[str] = None
    name2: Optional[str] = None
    opening: Optional[str] = None
    phone: Optional[str] = None
    skypeID: Optional[str] = None
    status: int
    userId: int
    website: Optional[str] = None


class CustomerPaymentInformation(BaseModel):
    accountHolder: Optional[str] = None
    accountID: Optional[int] = None
    BIC: Optional[str] = None
    contractNumber: Optional[str] = None
    debitAccount: Optional[str] = None
    IBAN: Optional[str] = None
    paymentMethodID: Optional[int] = None
    preselectedTaxID: Optional[int] = None
    salesTaxID: Optional[str] = None


class CustomerContactObject(BaseModel):
    addressID: int
    costCenter: Optional[str] = None
    customerContactID: int
    customerID: int
    email: Optional[str] = None
    externalID: Optional[str] = None
    fax: Optional[str] = None
    mobilePhone: Optional[str] = None
    name1: Optional[str] = None
    name2: Optional[str] = None
    phone: Optional[str] = None
    status: ContactPersonStatus
    supervisor1: Optional[str] = None
    supervisor2: Optional[str] = None
    userId: int


class CustomerContactIN(BaseModel):
    addressID: Optional[int] = None
    costCenter: Optional[str] = None
    customerContactID: Optional[int] = None
    customerID: Optional[int] = None
    email: Optional[str] = None
    externalID: Optional[str] = None
    fax: Optional[str] = None
    mobilePhone: Optional[str] = None
    name1: Optional[str] = None
    name2: Optional[str] = None
    phone: Optional[str] = None
    status: Optional[ContactPersonStatus] = None
    supervisor1: Optional[str] = None
    supervisor2: Optional[str] = None
    userId: Optional[int] = None


class InvoiceObject(BaseModel):
    briefDescription: Optional[str] = None
    currencyCode: Optional[str] = None
    customerID: int
    gross: float
    invoiceDate: datetime
    invoiceDocuments: List[int] = Field(default_factory=list)
    invoiceID: int
    invoiceNr: str
    net: float
    orderIDs: List[int] = Field(default_factory=list)
    outgoing: float
    paid: float
    status: InvoiceStatusType
    subject: Optional[str] = None
    tax: float
    valueDate: datetime



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..client import PlunetClient


class DataAdmin30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_available_languages(self, interface_lang="en"):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getAvailableLanguages,
            interface_lang,
        )

    def get_available_services(self, interface_lang="en"):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getAvailableServices, interface_lang
        )

    def get_available_countries(self, interface_lang="en"):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getAvailableCountries,
            interface_lang,
        )

    def get_available_document_templates(self):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getAvailableDocumentTemplates, None
        )

    def get_company_code_list(self):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getCompanyCodeList, None
        )

    def get_domestic_currency(self):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getDomesticCurrency, None
        )

    def get_list_of_registered_callbacks(self):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getListOfRegisteredCallbacks, None
        )

    def get_system_currencies(self):
        return self.__client.make_request(
            self.__client.plunet_server.DataAdmin30.getSystemCurrencies, None
        )



from typing import Callable, Union

from plunetapi import PlunetAPI

from .services import (
    DataAdmin30,
    DataCustomer30,
    DataCustomerContact30,
    DataDocument30,
    DataItem30,
    DataJob30,
    DataOrder30,
    DataOutgoingInvoice30,
    DataQuote30,
    DataRequest30,
    DataUser30,
    ReportCustomer30,
    ReportJob30,
)

from .exceptions import NoPlunetSession, PlunetAuthFailed, PlunetException


class PlunetClient:
    def __init__(self, base_url=None):
        self.plunet_server = PlunetAPI(base_url=base_url)
        self.uuid = None
        self.document = DataDocument30(self)
        self.user = DataUser30(self)
        self.item = DataItem30(self)
        self.order = DataOrder30(self)
        self.request = DataRequest30(self)
        self.quote = DataQuote30(self)
        self.admin = DataAdmin30(self)
        self.job = DataJob30(self)
        self.customer = DataCustomer30(self)
        self.customer_contact = DataCustomerContact30(self)
        self.outgoing_invoice = DataOutgoingInvoice30(self)
        self.report_customer = ReportCustomer30(self)
        self.report_job = ReportJob30(self)

    def login(self, username, password) -> None:
        uuid = self.plunet_server.PlunetAPI.login(username, password)
        if uuid == "refused":
            raise PlunetAuthFailed("Plunet authentication failed.")
        else:
            self.uuid = uuid

    def logout(self) -> None:
        if self.uuid is not None:
            self.plunet_server.PlunetAPI.logout(self.uuid)
            self.uuid = None
        return

    def make_request(
        self,
        operation_proxy: Callable,
        argument: Union[dict, str, int, list, None],
        unpack_dict: bool = False,
    ):
        if self.uuid is None:
            raise NoPlunetSession("You have no active Plunet session. Please log in.")

        if unpack_dict is True:
            result = operation_proxy(self.uuid, **argument)
        elif argument is not None:
            result = operation_proxy(self.uuid, argument)
        else:
            result = operation_proxy(self.uuid)

        if result.statusCode != 0:
            raise PlunetException(result.statusMessage)
        else:
            try:
                return result.data
            except:
                return result

    def make_download(
        self,
        operation_proxy: Callable,
        argument: Union[dict, str, int, list],
        unpack_dict: bool = False,
    ):
        if self.uuid is None:
            raise NoPlunetSession("You have no active Plunet session. Please log in.")

        if unpack_dict is True:
            result = operation_proxy(self.uuid, **argument)
        else:
            result = operation_proxy(self.uuid, argument)
        if result.statusCode != 0:
            raise Exception(result.statusMessage)
        else:
            try:
                return result.fileContent
            except:
                return result



from __future__ import annotations
from datetime import datetime
from typing import List, Union

from ..models import SearchFilter_Invoice, SelectionEntry_TimeFrame

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..client import PlunetClient

from ..enums import CatType, InvoiceStatusType, ProjectType, TimeFrameRelation_Invoice


class DataOutgoingInvoice30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def search_for_invoice_ids_by_invoice_date(
        self, from_date: datetime, to_date: datetime, status: InvoiceStatusType
    ) -> List[int]:

        timeframe = SelectionEntry_TimeFrame(
            dateFrom=from_date,
            dateTo=to_date,
            dateRelation=TimeFrameRelation_Invoice.INVOICE_DATE,
        )

        searchfilter_invoice = SearchFilter_Invoice(
            timeFrame=timeframe, invoiceStatus=status
        )
        return self.__client.make_request(
            self.__client.plunet_server.DataOutgoingInvoice30.search,
            searchfilter_invoice.dict(),
        )

    def get_invoice_object(self, invoice_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataOutgoingInvoice30.getInvoiceObject,
            invoice_id,
        )

    def get_invoice_company_code(self, invoice_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataOutgoingInvoice30.getCompanyCode, invoice_id
        )

    def get_invoice_receivable_account(self, invoice_id: int) -> str:
        return self.__client.make_request(
            self.__client.plunet_server.DataOutgoingInvoice30.getReceivableAccount,
            invoice_id,
        )

    def get_invoice_revenue_account(self, invoice_id: int) -> str:
        return self.__client.make_request(
            self.__client.plunet_server.DataOutgoingInvoice30.getRevenueAccount,
            invoice_id,
        )

    def get_invoice_payment_due_date(self, invoice_id: int) -> datetime:
        return self.__client.make_request(
            self.__client.plunet_server.DataOutgoingInvoice30.getPaymentDueDate,
            invoice_id,
        )



from enum import Enum


class SearchSelection_Resource(Enum):
    AGENCY = 9
    RESOURCE = 1


class FolderType(Enum):
    CUSTOMER = 19
    INVOICE = 13
    ORDER_FINAL = 12
    ORDER_ITEM_CAT = 8
    ORDER_ITEM_REFERENCE = 17
    ORDER_ITEM_SOURCE = 16
    ORDER_JOB_IN = 22
    ORDER_JOB_OUT = 23
    ORDER_OUT = 10
    ORDER_PRM = 25
    ORDER_REFERENCE = 5
    ORDER_SOURCE = 6
    QUOTE_FINAL = 11
    QUOTE_ITEM_CAT = 7
    QUOTE_ITEM_REFERENCE = 15
    QUOTE_ITEM_SOURCE = 14
    QUOTE_JOB_IN = 20
    QUOTE_JOB_OUT = 21
    QUOTE_OUT = 9
    QUOTE_PRM = 24
    QUOTE_REFERENCE = 3
    QUOTE_SOURCE = 4
    REQUEST_REFERENCE = 1
    REQUEST_SOURCE = 2
    RESOURCE = 18


class InvoiceType(Enum):
    CREDIT_NOTES = 2
    PAYABLES = 3
    RECEIVABLES = 1


class WorkingStatus(Enum):
    EXTERNAL = 2
    INTERNAL = 1


class QuoteStatusType(Enum):
    ACCEPTED = 7
    CANCELED = 5
    CHANGE_INTO_ORDER = 4
    CHECK_CLEARANCE = 11
    EXPIRED = 6
    IN_PREPARATION = 9
    NEW = 8
    PENDING = 1
    REJECTED = 3
    REVISED = 2


class Format(Enum):
    ABBREVIATED_VERSION = 1
    OUTPUT_AS_IN_ORDER = 6
    QUANTITY_NOT_WEIGHTED = 4
    STANDARD = 0
    STANDARD_WITH_PRICE_MEMO = 5
    TM_DISCONT = 3
    WEIGHTED_QUANTITY = 2


class WorkflowType(Enum):
    ORDER = 1
    QUOTE_ORDER = 2
    STANDARD = 0


class AddressType(Enum):
    DELIVERY = 1
    INVOICE = 2
    OTHER = 3


class JobRequestRankingMethod(Enum):
    JOB_FEEDBACK = 3
    JOBS_FOR_CUSTOMER = 2
    PRICE = 4
    RANKING_RESOURCE_ASSESSMENT = 1


class FormOfAddressType(Enum):
    COMPANY = 3
    MADAM = 2
    SIR = 1


class JobRoundAssignmentMethod(Enum):
    FIRST_COME_FIRST_SERVE = 2
    MANUAL = 1
    TOP_RANKED = 4


class TimeFrameRelation_Quote(Enum):
    EVENT_END_DATE = 9
    EVENT_START_DATE = 8
    INSTRUCTED_DATE = 10
    ITEM_CREATION_DATE = 2
    ITEM_DELIVERY_DATE = 4
    QUOTE_BINDING_DEADLINE_DATE = 6
    QUOTE_CREATION_DATE = 1
    QUOTE_DELIVERY_DATE = 3
    REJECTION_DATE = 7
    REQUEST_DATE = 5


class DocumentStatus(Enum):
    DOCUMENTS_APPROVED = 7
    DOCUMENTS_AVAILABLE = 1
    DOCUMENTS_DOWNLOADED = 2
    DOCUMENTS_IN_REVIEW = 3
    DOCUMENTS_RE_DELIVERED = 6
    NO_DOCUMENTS_AVAILABLE = 0
    POSTPROCESSING_IN_PROGRESS = 5
    POSTPROCESSING_REQUESTED = 4


class APIVersion(Enum):
    Version_10 = "Version_10"
    Version_25 = "Version_25"
    Version_30 = "Version_30"


class CurrencyType(Enum):
    HOMECURRENCY = 2
    PROJECTCURRENCY = 1


class CompanyCodeType(Enum):
    INVOICE = 1
    RIGHTS = 2


class CustomerType(Enum):
    DIRECT = 0
    DIRECT_INDIRECT = 1
    INDIRECT = 2


class JobRoundStatus(Enum):
    ASSIGNMENT_ERROR = 2
    CANCELED = 5
    IN_PREPARATION = 0
    JOB_ASSIGNED = 3
    NO_ASSIGNMENT = 4
    REACTION_TIME_ELAPSED = 6
    REQUESTED = 1


class CatType(Enum):
    ACROSS = 5
    CATALYST = 14
    DEJAVU = 12
    FALCON = 17
    FUSION = 11
    HELIUM = 15
    IDIOM = 9
    LOGOPORT = 8
    MEMOQ = 4
    MEMSOURCE = 16
    PASSOLO = 7
    PRACTICOUNT = 6
    TRADOS = 1
    TRANSIT = 3
    WORDFAST = 13
    XTM = 10


class SearchSelection_TeamMember(Enum):
    PROJECT_MANAGER = 4
    RESOURCE = 1
    SUPERVISOR = 8


class ItemStatus(Enum):
    ACCEPTED = 12
    APPROVED = 3
    CANCELED = 5
    DELIVERABLE = 7
    DELIVERED = 2
    IN_PREPERATION = 8
    IN_PROGRESS = 1
    INVOICED = 4
    NEW_AUTO = 6
    PAID = 9
    PENDING = 11
    REJECTED = 13
    SUM = 14
    WITHOUT_INVOICE = 10


class ArchivStatus(Enum):
    ACTIVE = 1
    ARCHIVED = 3
    COMPLETED = 6
    COMPLETED_ARCHIVABLE = 2
    IN_PREPARATION = 5
    QUOTE_MOVED_TO_ORDER = 4


class JobRoundAssignmentLimitType(Enum):
    ALL = 0
    LIMIT_X = 1
    MANUAL_SELECTED = 2


class SearchSelection_Customer(Enum):
    ACCOUNT_MANAGER = 7
    CUSTOMER = 1
    INDIRECT_CUSTOMER = 3


class TimeFrameRelation_Request(Enum):
    REQUEST_DATE = 1


class PayableStatus(Enum):
    CANCELED = 3
    CREATED_BY_EXTERNAL_USER = 4
    IN_PREPARATION = 5
    INVOICE_CHECKED = 6
    OUTSTANDING = 1
    PAID = 2


class RequestStatusType(Enum):
    CANCELED = 5
    CHANGED_INTO_ORDER = 7
    CHANGED_INTO_QUOTE = 6
    IN_PREPARATION = 1
    NEW_AUTO = 8
    PENDING = 2
    REJECTED = 9


class ResourceStatus(Enum):
    ACTIVE = 1
    BLOCKED = 3
    DELETION_REQUESTED = 10
    DISQUALIFIED = 9
    NEW = 4
    NEW_AUTO = 6
    NOT_ACTIVE_OR_OLD = 2
    PREMIUM = 5
    PROBATION = 7
    QUALIFIED = 8


class ProjectType(Enum):
    ORDER = 3
    QUOTE = 1


class PropertyUsageArea(Enum):
    CUSTOMER = 1
    ORDER = 6
    ORDER_ITEM = 10
    ORDER_JOB = 12
    QUOTE = 5
    QUOTE_ITEM = 9
    QUOTE_JOB = 11
    REQUEST = 4
    RESOURCE = 2


class ResourceType(Enum):
    PROJECT_MANAGER = 2
    RESOURCES = 0
    SUPERVISOR = 3
    TEAM_MEMBER = 1


class CustomerStatus(Enum):
    ACTIVE = 1
    AQUISITION_ADDRESS = 6
    BLOCKED = 5
    CONTACTED = 3
    DELETION_REQUESTED = 8
    NEW = 4
    NEW_AUTO = 7
    NOT_ACTIVE = 2


class JobStatus(Enum):
    APPROVED = 3
    ASSIGNED_WAITING = 7
    CANCELED = 4
    DELIVERED = 2
    IN_PREPERATION = 0
    IN_PROGRESS = 1
    INVOICE_ACCEPTED = 5
    INVOICE_CHECKED = 9
    INVOICE_CREATED = 10
    OVERDUE = 13
    PAYED = 6
    REQUESTED = 8
    TRANSFERRED_TO_ORDER = 12
    WITHOUT_INVOICE = 11


class CallbackType(Enum):
    NOTIFY = 2
    OBSERVER = 1


class WorkflowStatus(Enum):
    CANCELED = 2
    INPREPARATION = 0
    RELEASED = 1
    RELEASED_FOR_SELECTION = 3


class SearchScope(Enum):
    DATE_OF_ORDER = 1
    END_dATE_EVENT = 8
    INSTALLMENT_DATE = 6
    ITEM_CREATION_DATE = 2
    ITEM_DELIVERED_ON = 5
    ITEM_DUE_DATE = 4
    ORDER_CLOSING_DATE = 9
    ORDER_DUE_DATE = 3
    START_DATE_EVENT = 7


class TaxType(Enum):
    INFO = 3
    INFO_SUM = 6
    PRICE_BLOCK = 12
    SUM = 4
    TAX_1 = 0
    TAX_1_2 = 5
    TAX_1_2_3 = 8
    TAX_1_2_3_4 = 14
    TAX_1_2_3_4_5 = 14
    TAX_1_3 = 10
    TAX_1_4 = 17
    TAX_2 = 1
    TAX_2_3 = 11
    TAX_2_4_5 = 16
    TAX_3 = 7
    TAX_4 = 9
    TAX_5 = 13
    WITHOUT_TAX = 2


class TimeFrameRelation_Invoice(Enum):
    INVOICE_DATE = 1
    PAYABLE_UNTIL = 3
    PAYED = 4
    VALUTA = 2


class ExportedType(Enum):
    BOTH = 3
    EXPORTED = 1
    NOT_EXPORTED = 2


class JobActionLink(Enum):
    ACCEPT_JOB = 1
    DECLINE_JOB = 2
    DOWNLOADLINK_DOCUMENTS_JOB = 4
    JUMP_INTO_JOB = 3


class TextModuleType(Enum):
    DATE_FIELD = 3
    HYPER_LINK = 7
    LIST_BOX = 2
    MEMO_FIELD = 4
    MEMO_HISTORY_FIELD = 5
    NUMBER_FIELD = 6
    TEXT_FIELD = 1


class InvoiceStatusType(Enum):
    CANCELED = 3
    CANCELLATION_VOUCHER = 7
    IN_PREPARATION = 5
    OUTSTANDING = 1
    PAID = 2
    REMINDER_CREATED = 4
    UNCOLLECTABLE = 6
    UNDEFINED = 0


class EventType(Enum):
    DELIVERY_DATE_CHANGED = 5
    ENTRY_DELETED = 3
    NEW_ENTRY_CREATED = 2
    START_DATE_CHANGED = 4
    STATUS_CHANGED = 1


class ContactPersonStatus(Enum):
    ACTIVE = 1
    CONTACTED = 3
    DELETION_REQUESTED = 4
    NOT_ACTIVE = 2


class TextModuleUsageArea(Enum):
    CUSTOMER = 1
    INVOICE = "INVOICE"
    ORDER = 6
    ORDER_CUSTOMER_LOGIN = 19
    ORDER_JOB = 11
    PAYMENT = 8
    QUOTE = 5
    QUOTE_CUSTOMER_LOGIN = 18
    QUOTE_JOB = 10
    RECEIVABLES = 7
    RECEIVABLES_CREDIT_NOTE = 9
    REQUEST = 4
    REQUEST_CUSTOMER_LOGIN = 17
    RESOURCE = 2
    VENDOR = 3


class ContactType(Enum):
    CUSTOMER = 1
    MISCELLANEOUS = 3
    RESOURCE = 2


class CreditNoteStatus(Enum):
    CANCELLATION_VOUCHER = 5
    CANCELLED = 4
    CLEARED = 3
    IN_PREPARATION = 1
    OPEN = 2
    PAID = 6


class ProjectManagementType(Enum):
    MEMBER_OF_PROJECT_TEAM = 1
    PROJECT_MANAGEMENT = 0
    PROJECT_MANAGER = 2
    SUPERVISOR = 3


class PropertyType(Enum):
    MULTI_SELECT = 2
    SINGLE_SELECT = 1


class OrderActionLink(Enum):
    JUMP_INTO_ORDER = 1
    JUMP_INTO_ORDER_ASSET = 2


class ProjectClassType(Enum):
    ALL = 0
    INTERPRETING = 2
    TRANSLATION = 1



from __future__ import annotations
from typing import List, Union

from zeep.helpers import serialize_object

from ..models import (
    CustomerContactIN,
    CustomerContactObject,
    CustomerObject,
    CustomerPaymentInformation,
    SearchFilter_Customer,
)

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..client import PlunetClient


class DataCustomer30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_customer_object(self, customer_id: int) -> CustomerObject:
        return self.__client.make_request(
            self.__client.plunet_server.DataCustomer30.getCustomerObject, customer_id
        )

    def get_customer_payment_information(
        self, customer_id: int
    ) -> CustomerPaymentInformation:
        data = self.__client.make_request(
            self.__client.plunet_server.DataCustomer30.getPaymentInformation,
            customer_id,
        )
        return CustomerPaymentInformation(**serialize_object(data, dict))


class DataCustomerContact30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_all_customer_contact_objects(
        self, customer_id: int
    ) -> List[CustomerContactObject]:
        """
        :param customer_id: int
        :return: List of customer objects
        """
        return self.__client.make_request(
            self.__client.plunet_server.DataCustomerContact30.getAllContactObjects,
            customer_id,
        )

    def get_all_customer_contact_ids(self, customer_id: int) -> List[int]:
        """
        :param customer_id: int
        :return: list of customer IDs
        """
        return self.__client.make_request(
            self.__client.plunet_server.DataCustomerContact30.getAllContacts,
            customer_id,
        )

    def get_customer_contact_object(self, contact_id: int):
        return self.__client.make_request(
            self.__client.plunet_server.DataCustomerContact30.getContactObject,
            contact_id,
        )

    def update_customer_contact(self, contact_in: CustomerContactIN):

        arg = {"CustomerContact": contact_in.dict(), "enabled": False}
        return self.__client.make_request(
            self.__client.plunet_server.DataCustomerContact30.update,
            arg,
            unpack_dict=True,
        )


class ReportCustomer30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def search_for_customers(
        self, search_filter: Union[SearchFilter_Customer, dict]
    ) -> list:
        if type(search_filter) != SearchFilter_Customer:
            arg = SearchFilter_Customer(**search_filter).dict()
        else:
            arg = search_filter.dict()
        return self.__client.make_request(
            self.__client.plunet_server.ReportCustomer30.search, arg
        )



from __future__ import annotations
from typing import List, Union

from ..models import SearchFilter_Job

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..client import PlunetClient

from ..enums import CatType, ProjectType


class DataJob30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def get_jobtype_shortname(
        self, job_id: int, project_type: ProjectType = ProjectType.ORDER
    ):
        arg = {"jobID": job_id, "projectType": project_type.value}
        return self.__client.make_request(
            self.__client.plunet_server.DataJob30.getJobType_ShortName,
            arg,
            unpack_dict=True,
        )

    def get_job_list_of_item_for_view(
        self, item_id: int, project_type: ProjectType = ProjectType.ORDER
    ):
        arg = {"projectType": project_type.value, "itemID": item_id}
        return self.__client.make_request(
            self.__client.plunet_server.DataJob30.getJobListOfItem_ForView,
            arg,
            unpack_dict=True,
        )

    def set_cat_report_to_job(
        self,
        job_id: int,
        byte_stream: bytes,
        file_name: str,
        file_size: int,
        analyze_copy: bool,
        cat_type: CatType = CatType.MEMSOURCE,
        project_type: ProjectType = ProjectType.ORDER,
    ):

        arg = {
            "FileByteStream": byte_stream,
            "FilePathName": file_name,
            "Filesize": file_size,
            "catType": cat_type.value,
            "analyzeAndCopyResultToJob": analyze_copy,
            "projectType": project_type.value,
            "jobID": job_id,
        }

        return self.__client.make_request(
            self.__client.plunet_server.DataJob30.setCatReport2, arg, unpack_dict=True
        )


class ReportJob30:
    def __init__(self, client: PlunetClient):
        self.__client = client

    def search_for_jobs(
        self, search_filter: Union[SearchFilter_Job, dict]
    ) -> List[int]:
        if type(search_filter) != SearchFilter_Job:
            arg = SearchFilter_Job(**search_filter).dict()
        else:
            arg = search_filter.dict()

        return self.__client.make_request(
            self.__client.plunet_server.ReportJob30.search, arg
        )
